"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCacheDirs = getCacheDirs;
exports.saveCache = saveCache;
exports.restoreCache = restoreCache;
exports.normalizedCacheDir = normalizedCacheDir;
const path_1 = __importDefault(require("path"));
const process_1 = __importDefault(require("process"));
const config_1 = require("./config");
function getCacheDirs(publish) {
    return [publish, normalizedCacheDir(publish)];
}
async function saveCache({ publish, utils, }) {
    if (process_1.default.env.NETLIFY_LOCAL) {
        return;
    }
    const cacheDirs = getCacheDirs(publish);
    if (await utils.cache.save(cacheDirs, { move: true })) {
        utils.status.show({
            title: 'Essential Gatsby Build Plugin ran successfully',
            summary: 'Stored the Gatsby cache to speed up future builds. 🔥',
        });
    }
    else {
        console.log('No Gatsby build found.');
    }
}
async function restoreCache({ publish, utils, }) {
    if (process_1.default.env.NETLIFY_LOCAL) {
        return;
    }
    const cacheDirs = getCacheDirs(publish);
    if (await utils.cache.restore(cacheDirs, { move: true })) {
        console.log('Found a Gatsby cache. We’re about to go FAST. ⚡️');
    }
    else {
        console.log('No Gatsby cache found. Building fresh.');
    }
}
function normalizedCacheDir(publish) {
    return path_1.default.join((0, config_1.getGatsbyRoot)(publish), `.cache`);
}
