"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteFunctions = exports.setupImageCdn = exports.writeFunctions = void 0;
const fs_extra_1 = require("fs-extra");
const pathe_1 = require("pathe");
const handlers_1 = require("../templates/handlers");
const config_1 = require("./config");
const image_cdn_1 = require("./image_cdn");
const writeFunction = async ({ renderMode, handlerName, appDir, functionsSrc, }) => {
    const source = (0, handlers_1.makeHandler)(appDir, renderMode);
    await (0, fs_extra_1.ensureDir)((0, pathe_1.join)(functionsSrc, handlerName));
    await (0, fs_extra_1.writeFile)((0, pathe_1.join)(functionsSrc, handlerName, `${handlerName}.js`), source);
    await (0, fs_extra_1.copyFile)((0, pathe_1.join)(__dirname, '..', '..', 'lib', 'templates', 'utils.js'), (0, pathe_1.join)(functionsSrc, handlerName, 'utils.js'));
};
const writeApiFunction = async ({ appDir, functionDir }) => {
    const source = (0, handlers_1.makeApiHandler)(appDir);
    // This is to ensure we're copying from the compiled js, not ts source
    await (0, fs_extra_1.copy)((0, pathe_1.join)(__dirname, '..', '..', 'lib', 'templates', 'api'), functionDir);
    await (0, fs_extra_1.writeFile)((0, pathe_1.join)(functionDir, '__api.js'), source);
};
const writeFunctions = async ({ constants, netlifyConfig, neededFunctions, prepareImageCdnResult, }) => {
    const { PUBLISH_DIR, INTERNAL_FUNCTIONS_SRC } = constants;
    const siteRoot = (0, config_1.getGatsbyRoot)(PUBLISH_DIR);
    const functionDir = (0, pathe_1.resolve)(INTERNAL_FUNCTIONS_SRC, '__api');
    const appDir = (0, pathe_1.relative)(functionDir, siteRoot);
    await (0, fs_extra_1.ensureDir)(INTERNAL_FUNCTIONS_SRC);
    if (neededFunctions.includes('SSR')) {
        await writeFunction({
            renderMode: 'SSR',
            handlerName: '__ssr',
            appDir,
            functionsSrc: INTERNAL_FUNCTIONS_SRC,
        });
    }
    if (neededFunctions.includes('DSG')) {
        await writeFunction({
            renderMode: 'DSG',
            handlerName: '__dsg',
            appDir,
            functionsSrc: INTERNAL_FUNCTIONS_SRC,
        });
    }
    if (prepareImageCdnResult &&
        prepareImageCdnResult.imageCDNImplementation !== 'NONE') {
        await (0, exports.setupImageCdn)({ constants, netlifyConfig, prepareImageCdnResult });
    }
    if (neededFunctions.includes('API')) {
        await writeApiFunction({ appDir, functionDir });
    }
};
exports.writeFunctions = writeFunctions;
// eslint-disable-next-line max-lines-per-function, complexity
const setupImageCdn = async ({ constants, netlifyConfig, prepareImageCdnResult, }) => {
    await (0, fs_extra_1.ensureDir)((0, pathe_1.join)(constants.INTERNAL_FUNCTIONS_SRC, '_ipx'));
    await (0, fs_extra_1.copy)((0, pathe_1.join)(__dirname, '..', '..', 'src', 'templates', 'ipx.ts'), (0, pathe_1.join)(constants.INTERNAL_FUNCTIONS_SRC, '_ipx', '_ipx.ts'), { overwrite: true });
    await (0, fs_extra_1.writeFile)((0, pathe_1.join)(constants.INTERNAL_FUNCTIONS_SRC, '_ipx', 'config.ts'), `exports.config = ${JSON.stringify(prepareImageCdnResult.imageCDNConfig)}`);
    if (prepareImageCdnResult.imageCDNImplementation ===
        image_cdn_1.ImageCdnImplementation.NETLIFY_IMAGE_CDN) {
        await (0, fs_extra_1.copyFile)((0, pathe_1.join)(__dirname, '..', '..', 'src', 'templates', 'image.ts'), (0, pathe_1.join)(constants.INTERNAL_FUNCTIONS_SRC, '__image.ts'));
        netlifyConfig.redirects.push(...[
            // QUERY_WITH_ENCRYPTED_URL is forcefully disable when using Netlify Image CDN
            // so we don't need to handle it here
            prepareImageCdnResult.imageCDNConfig.enabledUrlPatterns.includes(image_cdn_1.ImageCdnUrlSyntax.QUERY)
                ? {
                    from: '/_gatsby/image/:unused/:unused2/:filename',
                    // eslint-disable-next-line id-length
                    query: { u: ':url', a: ':args', cd: ':cd' },
                    to: '/.netlify/functions/__image/image_query_compat?url=:url&args=:args&cd=:cd',
                    status: 301,
                    force: true,
                }
                : undefined,
            prepareImageCdnResult.imageCDNConfig.enabledUrlPatterns.includes(image_cdn_1.ImageCdnUrlSyntax.BASE64)
                ? {
                    from: '/_gatsby/image/*',
                    to: '/.netlify/functions/__image',
                    status: 200,
                    force: true,
                }
                : undefined,
        ].filter(Boolean));
    }
    else if (prepareImageCdnResult.imageCDNImplementation === image_cdn_1.ImageCdnImplementation.IPX) {
        netlifyConfig.redirects.push(
        // eslint-disable-next-line no-sparse-arrays
        ...[
            prepareImageCdnResult.imageCDNConfig.enabledUrlPatterns.includes(image_cdn_1.ImageCdnUrlSyntax.QUERY_WITH_ENCRYPTED_URL)
                ? {
                    from: `/_gatsby/image/:unused/:unused2/:filename`,
                    // eslint-disable-next-line id-length
                    query: { eu: ':encrypted_url', a: ':args' },
                    to: `/.netlify/builders/_ipx/image_query_compat_eu/:args/:encrypted_url/:filename`,
                    status: 301,
                    force: true,
                }
                : undefined,
            prepareImageCdnResult.imageCDNConfig.enabledUrlPatterns.includes(image_cdn_1.ImageCdnUrlSyntax.QUERY)
                ? {
                    from: `/_gatsby/image/:unused/:unused2/:filename`,
                    // eslint-disable-next-line id-length
                    query: { u: ':url', a: ':args' },
                    to: `/.netlify/builders/_ipx/image_query_compat/:args/:url/:filename`,
                    status: 301,
                    force: true,
                }
                : undefined,
            prepareImageCdnResult.imageCDNConfig.enabledUrlPatterns.includes(image_cdn_1.ImageCdnUrlSyntax.BASE64)
                ? {
                    from: '/_gatsby/image/*',
                    to: '/.netlify/builders/_ipx',
                    status: 200,
                    force: true,
                }
                : undefined,
            ,
        ].filter(Boolean));
    }
    netlifyConfig.redirects.push(...[
        prepareImageCdnResult.imageCDNConfig.enabledUrlPatterns.includes(image_cdn_1.ImageCdnUrlSyntax.QUERY_WITH_ENCRYPTED_URL)
            ? {
                from: `/_gatsby/file/:unused/:filename`,
                query: { eu: ':encrypted_url' },
                to: `/.netlify/functions/_ipx/file_query_compat_eu/:encrypted_url/:filename`,
                status: 301,
                force: true,
            }
            : undefined,
        prepareImageCdnResult.imageCDNConfig.enabledUrlPatterns.includes(image_cdn_1.ImageCdnUrlSyntax.QUERY)
            ? {
                from: `/_gatsby/file/:unused/:filename`,
                // eslint-disable-next-line id-length
                query: { u: ':url' },
                to: `/.netlify/functions/_ipx/file_query_compat/:url/:filename`,
                status: 301,
                force: true,
            }
            : undefined,
        prepareImageCdnResult.imageCDNConfig.enabledUrlPatterns.includes(image_cdn_1.ImageCdnUrlSyntax.BASE64)
            ? {
                from: '/_gatsby/file/*',
                to: '/.netlify/functions/_ipx',
                status: 200,
                force: true,
            }
            : undefined,
    ].filter(Boolean));
};
exports.setupImageCdn = setupImageCdn;
const deleteFunctions = async ({ INTERNAL_FUNCTIONS_SRC, }) => {
    for (const func of ['__api', '__ssr', '__dsg']) {
        const funcDir = (0, pathe_1.resolve)(INTERNAL_FUNCTIONS_SRC, func);
        if ((0, fs_extra_1.existsSync)(funcDir)) {
            await (0, fs_extra_1.rm)(funcDir, { recursive: true, force: true });
        }
    }
};
exports.deleteFunctions = deleteFunctions;
/* eslint-enable max-lines */
